package xyz.veronie.bgg.ui.filters;

import java.util.HashMap;

import javax.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import xyz.veronie.bgg.data.EventConstants;
import xyz.veronie.bgg.data.FilterFlagState;
import xyz.veronie.bgg.data.UserFlag;





/// These are the controls to retrieve thing IDs for a given BGG user
/// Filters can be set for the state of the thing in the user's collection
/// (i.e. owned, previously owned, wishlist, etc.)
public class BggUserSourceFilter {
	
	@Inject
	private IEventBroker eventBroker;
	
	private HashMap<UserFlag, FilterFlagState> flags = new HashMap<UserFlag, FilterFlagState>() {
		private static final long serialVersionUID = 1L;
	{
		put(UserFlag.OWN, FilterFlagState.IS);
		put(UserFlag.PREVIOUSLY_OWNED, FilterFlagState.ISNOT);
		put(UserFlag.FOR_TRADE, FilterFlagState.IGNORE);
		put(UserFlag.WANTED, FilterFlagState.IGNORE);
		put(UserFlag.WTP, FilterFlagState.IGNORE);
		put(UserFlag.WTB, FilterFlagState.IGNORE);
		put(UserFlag.WISHLIST, FilterFlagState.IGNORE);
		put(UserFlag.PREORDERED, FilterFlagState.IGNORE);
	}};
	
	public void create(Composite parent, int style) {

        GridLayout filterLayout = new GridLayout(4, false);
        parent.setLayout(filterLayout);
        
        Label lblUser = new Label(parent, SWT.LEFT);
        lblUser.setText("User name: ");
        lblUser.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		        
        Combo cbUserName = new Combo(parent, SWT.DROP_DOWN);
        cbUserName.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
        cbUserName.addSelectionListener(new SelectionAdapter() {
            public void widgetDefaultSelected(SelectionEvent e) {
                setUser(cbUserName.getText());
            }
        });
        cbUserName.addFocusListener(new FocusListener() {
			
			@Override
			public void focusLost(FocusEvent e) {
				setUser(cbUserName.getText());
			}
			
			@Override
			public void focusGained(FocusEvent e) {
				// nothing
			}
		});
        
        Button btEditUsers = new Button(parent, SWT.PUSH);
        btEditUsers.setText("edit users");
        btEditUsers.setEnabled(false);
        btEditUsers.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
        // TODO: implement edit users

        Label infoLabel = new Label(parent, SWT.LEFT);
        infoLabel.setText("Select which flags are used as filter. Filters follow 'AND' rule.");
        GridData gdInfo = new GridData(SWT.FILL, SWT.FILL, true, false);
        gdInfo.horizontalSpan = 4;
        infoLabel.setLayoutData(gdInfo);

        makeFilter(parent, UserFlag.OWN);
        makeFilter(parent, UserFlag.WTP);
        
        makeFilter(parent, UserFlag.PREVIOUSLY_OWNED);
        makeFilter(parent, UserFlag.WTB);
        
        makeFilter(parent, UserFlag.FOR_TRADE);
        makeFilter(parent, UserFlag.WISHLIST);
        
        makeFilter(parent, UserFlag.WANTED);
        makeFilter(parent, UserFlag.PREORDERED);
        
        parent.layout();
	}
	
	
	/// Create a control for a filter. 
	// Filters are tri-state: include iff has flag, include iff it does not have the flag, 
	// don't care if it has that flag.
	// TODO: create a three-way toggle button instead. 
	private void makeFilter(Composite parent, UserFlag key) {
		Combo filterCombo = new Combo(parent, SWT.READ_ONLY);
		filterCombo.add("is");
		filterCombo.add("is not");
		filterCombo.add("ignored");
		filterCombo.select(flags.get(key).get());
		filterCombo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		// TODO: add listener - send a CONFIG_CHANGED event when something changes
		
		Label filterLabel = new Label(parent, SWT.LEFT);
		filterLabel.setText(key.toString());
		filterLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
	}

	public void setUser(String user) {
		if(eventBroker != null) {
			eventBroker.post(EventConstants.TOPIC_USER_CHANGED, user);
		} else {
		   	System.out.println("null???");
		}
	}
	
}
